/*
* @par License info
*
*
* Copyright (C) 2016 Carlos Fernández Moraga
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* This is free software, and you are welcome to redistribute it.
*
*/

package es.promax.bluetooth;

import android.app.Activity;
import android.app.ListActivity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.Set;

import es.promax.promaxbt.R;

public class ListadoDeviceBT extends ListActivity {
	
	public static final int REQUEST_ENABLE_BT = 7;

	private ArrayAdapter<String> mArrayAdapter;
	private BluetoothAdapter mBluetoothAdapter;
	private ArrayList<BluetoothDevice> btDeviceArray = new ArrayList<BluetoothDevice>();
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.listado_device_bt);
		
		mArrayAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1);
		setListAdapter(mArrayAdapter);
	}

    @Override
	public  void onStart() {
    super.onStart();
    ConfigBT();
    }
    
    @Override
    public void onDestroy(){
    	 super.onDestroy();
    	 //if (mBluetoothAdapter != null) mBluetoothAdapter.s;//Detenemos servicio
    }
	
        
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
	
		BluetoothDevice device = btDeviceArray.get(position);
		
		Intent intent = new Intent();
		intent.putExtra("deviceMAC", device.getAddress());
		setResult(RESULT_OK, intent);
		finish();	
	}
	
    public void ConfigBT(){
    	// Obtenemos el adaptador de bluetooth
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        
        
        
		// Check smartphone support Bluetooth
		if(mBluetoothAdapter == null){
			//Device does not support Bluetooth
			Toast.makeText(getApplicationContext(), "Not support bluetooth", Toast.LENGTH_SHORT).show();
			finish();
		}else if(!mBluetoothAdapter.isEnabled()){	// Check Bluetooth enabled
		
			Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
		}else {
			//Clean
			mArrayAdapter.clear();
			// Queryng paried devices
			Set<BluetoothDevice> pariedDevices = mBluetoothAdapter.getBondedDevices();
			if(pariedDevices.size() > 0){
				for(BluetoothDevice device : pariedDevices){
					mArrayAdapter.add(device.getName() + "\n" + device.getAddress());
					btDeviceArray.add(device);
				}
			}
		}
        
        
    }
    


	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
  	  //Una vez que se ha realizado una actividad regresa un "resultado"...
  	      switch (requestCode) {
  	               
  	            case REQUEST_ENABLE_BT://Respuesta de intento de encendido de BT
  	                if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio
  	                	 ConfigBT();
  	                } else {//No se activo BT, salimos de la app                 
  	                    finish();}
		  }
	}
}

