/*
* @par License info
*
*
* Copyright (C) 2016 Carlos Fernández Moraga
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* This is free software, and you are welcome to redistribute it.
*
*/

package es.promax.pantallas;

import android.app.Activity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.Vibrator;
import android.support.v4.app.NotificationCompat;
import android.text.InputFilter;
import android.text.Spanned;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Locale;

import es.promax.bluetooth.ConexionBT;
import es.promax.bluetooth.ListadoDeviceBT;
import es.promax.canalizacion.Canalizacion;
import es.promax.inicio.PantallaInicioActivity;
import es.promax.promaxbt.DibujarLienzoConstelacion;
import es.promax.promaxbt.Promax12;
import es.promax.promaxbt.R;


public class ActivityPantallaContelacion extends Activity {

    public static final String TAG = "PX12";
    public static final boolean D = true;

    // Tipos de mensaje enviados y recibidos desde el Handler de ConexionBT
    public static final int MESSAGE_ESTADO_CAMBIADO = 1;
    public static final int MESSAGE_LEIDO = 2;
    public static final int MESSAGE_ESCRITO = 3;
    public static final int MESSAGE_NOMBRE_DISPOSITIVO = 4;
    public static final int MESSAGE_TOAST = 5;
    public static final int MESSAGE_DESCONECTADO = 6;
    public static final int MESSAGE_ERROR_CONEXION = 7;

    public static final int MENSAJE_RECIBIDO_NOVALIDO = 0;
    public static final int MENSAJE_RECIBIDO_ALARMA = 1;
    public static final int MENSAJE_RECIBIDO_BATERIA = 2;
    public static final int MENSAJE_RECIBIDO_TEXTO = 3;
    public static final int MENSAJE_RECIBIDO_VERSION = 4;
    public static final int MENSAJE_RECIBIDO_MESURA_SCAN = 5;
    public static final int MENSAJE_RECIBIDO_MESURA_SPAN = 6;
    public static final int MENSAJE_RECIBIDO_FREQUENCY = 7;
    public static final int MENSAJE_RECIBIDO_POINT_COSTELATION = 8;
    public static final int MENSAJE_RECIBIDO_DADES_COSTELLATION = 9;
    public static final int MENSAJE_RECIBIDO_BERS_COSTELLATION = 10;
    public static final int MENSAJE_RECIBIDO_MER_COSTELLATION = 11;
    public static final int MENSAJE_RECIBIDO_DBM_EQUIP = 12;



    //-----------------
    public static final String DEVICE_NAME = "device_name";
    public static final String TOAST = "toast";
    //-----------------
    public static final int REQUEST_ENABLE_BT = 107;
    public static final int REQUEST_LIST_BT = 108;
    public static final int REQUEST_INICIO_PANTALLA = 109;
    public static final int REQUEST_SETTINS_PANTALLA = 110;
    public static final int REQUEST_PAIR_DEVICE = 120;
    private static final int NOTIF_ID = 91372;
    Canalizacion canalesCCIR;
    private BluetoothDevice device;
    //Nombre del dispositivo conectado
    private String mConnectedDeviceName = null;
    private BluetoothAdapter AdaptadorBT = null;
    //Objeto miembro para el servicio de ConexionBT
    private ConexionBT Servicio_BT = null;
    private String deviceMAC = "00:16:A4:04:99:C4";
    //Vibrador
    private Vibrator vibrador;
    private Boolean activarNotificaciones = false;
    private Boolean inicioAplicacionPromax12 = true;
    private Boolean finAplicacionPromax12 = false;
    private NotificationManager mNotificationManager;
    private DibujarLienzoConstelacion lienzo;
    private TextView textPreBer;
    private TextView textPosBer;
    private TextView textTipoModulaciónPortadoras;
    private TextView textCodeRate;
    private TextView textPortadoras;
    private TextView textIntervaloGuarda;
    private TextView textMer;
    private EditText textFrequency;
    private Spinner canalizacion;
    private Promax12 px12 = new Promax12();
    final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {

            switch (msg.what) {
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                case MESSAGE_ESCRITO:
                    byte[] writeBuf = (byte[]) msg.obj;//buffer de escritura...
                    // Construye un String del Buffer
                    String writeMessage = new String(writeBuf);
                    if (D) Log.e(TAG, "Message_write  =w= " + writeMessage);
                    break;
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                case MESSAGE_LEIDO:
                    //byte[] readBuf = (byte[]) msg.obj;//buffer de lectura...
                    //Construye un String de los bytes validos en el buffer
                    // String readMessage = new String(readBuf, 0, msg.arg1);
                    String readMessage = (String) msg.obj;
                    if (msg.arg1 > 0) {
                        //  mainText.append(readMessage);
                        px12.mensajeRecibido(readMessage);
                        if (D) Log.e(TAG, "Message_read   =r= " + readMessage);
                    }
                    break;
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                case MESSAGE_NOMBRE_DISPOSITIVO:
                    mConnectedDeviceName = msg.getData().getString(DEVICE_NAME); //Guardamos nombre del dispositivo
                    Toast.makeText(getApplicationContext(), "Conectado con " + mConnectedDeviceName, Toast.LENGTH_SHORT).show();
                    conectadoBT();
                    break;
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                case MESSAGE_TOAST:
                    Toast.makeText(getApplicationContext(), msg.getData().getString(TOAST),
                            Toast.LENGTH_SHORT).show();
                    break;
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                case MESSAGE_DESCONECTADO:
                    if (D) Log.e("Conexion", "DESConectados");
                    desconectadoBT();
                    showNotification("Bluetooth desconectado");
                    break;
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                case MESSAGE_ERROR_CONEXION:
                    if (D) Log.e("Conexion", "Error Conexion");
                    desconectadoBT();
                    break;
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
            }//FIN DE SWITCH CASE PRIMARIO DEL HANDLER
        }//FIN DE METODO INTERNO handleMessage
    };//Fin de Handler


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);


        if (inicioAplicacionPromax12) {

            inicioPantallaPresentacion();
            inicioAplicacionPromax12 = false;

        }

        setContentView(R.layout.activity_constelation);


        textPreBer = (TextView) findViewById(R.id.textviewCber);
        textPosBer = (TextView) findViewById(R.id.textviewVber);
        textTipoModulaciónPortadoras = (TextView) findViewById(R.id.textTipoModulaciónPortadoras);
        textCodeRate = (TextView) findViewById(R.id.textviewCodeRate);
        textPortadoras = (TextView) findViewById(R.id.textviewPortadoras);
        textIntervaloGuarda = (TextView) findViewById(R.id.textviewIntervaloGuarda);
        textMer = (TextView) findViewById(R.id.textviewMer);

        textFrequency = (EditText) findViewById(R.id.editTextFrequecy);


        InputFilter filter = new InputFilter() {
            final int maxDigitsBeforeDecimalPoint = 4;
            final int maxDigitsAfterDecimalPoint = 2;

            @Override
            public CharSequence filter(CharSequence source, int start, int end,
                                       Spanned dest, int dstart, int dend) {
                StringBuilder builder = new StringBuilder(dest);
                builder.replace(dstart, dend, source
                        .subSequence(start, end).toString());
                if (!builder.toString().matches(
                        "(([1-9]{1})([0-9]{0," + (maxDigitsBeforeDecimalPoint - 1) + "})?)?(\\.[0-9]{0," + maxDigitsAfterDecimalPoint + "})?"

                )) {
                    if (source.length() == 0)
                        return dest.subSequence(dstart, dend);
                    return "";
                }

                return null;

            }
        };

        textFrequency.setFilters(new InputFilter[]{filter});

        canalesCCIR = new Canalizacion();

        ArrayAdapter<CharSequence> adapter =
                new ArrayAdapter<CharSequence>(this, R.layout.spinner_format_text);
        adapter.setDropDownViewResource(R.layout.spinner_format_text);


        canalizacion = (Spinner) findViewById(R.id.spinnerCanalizacion);


        for (int i = 0; i < canalesCCIR.getCanalizacion().size(); i++) {
            adapter.add(canalesCCIR.getCanalizacion().get(i).getNameChannel());
        }


        canalizacion.setAdapter(adapter);

        canalizacion.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {

                int frequencyKhz = canalesCCIR.getCanalizacion().get(pos).getFrequencyChannel();
                textFrequency.setText(String.format(Locale.ENGLISH, "%.2f", ((double) frequencyKhz / 100.00)));//String.valueOf((double)canalesCCIR.getCanalizacion().get(pos).getFrequencyChannel()/100.00));

                if (Servicio_BT != null) {
                    px12.setFrequencyKhz(frequencyKhz);

                    String auxEnviar = "*FR" + String.format("%06X", px12.getFrequencyKhz() & 0x0FFFFF) + "$";

                    sendMessage(auxEnviar);

                    try {
                        Thread.sleep(500);
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                    sendMessage("*SC30$");
                }

            }

            public void onNothingSelected(AdapterView<?> parent) {
            }
        });

        lienzo = new DibujarLienzoConstelacion(this, px12);


        //Mensajes barra de estado
        mNotificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

        FrameLayout layautDibujo = (FrameLayout) findViewById(R.id.frameLayoutLienzo);

        layautDibujo.addView(lienzo);


        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);


        startTimerThread();
    }

    private void startTimerThread() {
        final Handler handler = new Handler();

        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                while (true) {
                    try {
                        px12.gestionListaDeMensajesRecibido();
                        Thread.sleep(5);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            gestionarMensajeRecibido();
                        }
                    });
                }
            }
        };
        new Thread(runnable).start();
    }

    public void gestionarMensajeRecibido() {


        ArrayList<Integer> listadoMensajes = px12.getListadoMensajes();


        ArrayList<Integer> arrayListCopia = new ArrayList<Integer>(listadoMensajes);


        listadoMensajes.clear();


        while (arrayListCopia.remove(null)) {
        }


        if (!arrayListCopia.isEmpty()) {

            arrayListCopia.remove(null);

            for (Integer i : arrayListCopia) {

                //for (int i = 0; i < listadoMensajes.size(); ++i) {


                switch (i) {

                    case MENSAJE_RECIBIDO_BATERIA:
                        try {
                            lienzo.invalidate();
                        } catch (NumberFormatException ex) {
                            Log.d(TAG, "No es un número, es una cadena de texto.");
                        }
                        break;
                    case MENSAJE_RECIBIDO_TEXTO:
                        break;
                    case MENSAJE_RECIBIDO_VERSION:
                        break;
                    case MENSAJE_RECIBIDO_NOVALIDO:
                        break;
                    case MENSAJE_RECIBIDO_FREQUENCY:
                        textFrequency.setText(String.format(Locale.ENGLISH, "%.2f", ((double) px12.getFrequencyKhz() / 100.00)));
                        break;
                    case MENSAJE_RECIBIDO_ALARMA:
                        showNotification("Alarma activada");
                        lienzo.invalidate();
                        vibrador.vibrate(100);
                        break;
                    case MENSAJE_RECIBIDO_MESURA_SCAN:
                        px12.setInicializadoPromax12(true);
                        px12.setPantalla(MENSAJE_RECIBIDO_MESURA_SCAN);
                        lienzo.invalidate();
                        break;
                    case MENSAJE_RECIBIDO_MESURA_SPAN:
                        px12.setInicializadoPromax12(true);
                        px12.setPantalla(MENSAJE_RECIBIDO_MESURA_SPAN);
                        lienzo.invalidate();
                        break;

                    case MENSAJE_RECIBIDO_POINT_COSTELATION:
                        px12.setInicializadoPromax12(true);
                        px12.setPantalla(MENSAJE_RECIBIDO_POINT_COSTELATION);
                        lienzo.invalidate();
                        break;
                    case MENSAJE_RECIBIDO_DADES_COSTELLATION:
                        textPortadoras.setText(px12.getStringPortadoras());
                        textCodeRate.setText(px12.getStringCodeRate());
                        textIntervaloGuarda.setText(px12.getStringIntervaloGuarda());
                        textTipoModulaciónPortadoras.setText(px12.getStringTipoModulaciónPortadoras());
                        break;

                    case MENSAJE_RECIBIDO_BERS_COSTELLATION:
                        textPosBer.setText(px12.getPostBer());
                        textPreBer.setText(px12.getPreBer());
                        break;

                    case MENSAJE_RECIBIDO_MER_COSTELLATION:
                        textMer.setText(px12.getMer());
                        break;
                }
            }
            //La borramos
            arrayListCopia.clear();
            //arrayListCopia

        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        activarNotificaciones = false;
        Log.d(TAG, "NotificationTest: OnRestart");
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.d(TAG, "NotificationTest: OnResume");
    }

    @Override
    protected void onStop() {
        super.onStop();
        activarNotificaciones = true;
        Log.d(TAG, "NotificationTest: OnStop");
    }

    @Override
    public void onStart() {
        super.onStart();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        activarNotificaciones = false;
        finAplicacionPromax12 = true;

        mNotificationManager.cancel(NOTIF_ID);

        //salvarDatosGuardados();


        if (Servicio_BT != null) Servicio_BT.stop();//Detenemos servicio
        Servicio_BT = null;
        finish();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        if (id == R.id.action_settings) {
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    //-----  Bluetooth ------

    private void inicioPantallaPresentacion() {
        Intent intent = new Intent(ActivityPantallaContelacion.this, PantallaInicioActivity.class);
        startActivityForResult(intent, REQUEST_INICIO_PANTALLA);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        //Una vez que se ha realizado una actividad regresa un "resultado"...
        switch (requestCode) {

            case REQUEST_ENABLE_BT://Respuesta de intento de encendido de BT
                if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio
                    ConfigBT();

                } else {//No se activo BT, salimos de la app
                    Log.d(TAG, "Bluetooth: el usuario NO acepta encenderlo");
                    finish();
                }
                break;

            case REQUEST_INICIO_PANTALLA://Respuesta de intento de encendido de BT
                if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio
                    buscarBT();

                } else {//No se activo BT, salimos de la app
                    Log.d(TAG, "Error: Pantalla inicio");
                    finish();
                }
                break;

            case REQUEST_LIST_BT://Respuesta de intento de encendido de BT
                if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio
                    deviceMAC = data.getStringExtra("deviceMAC");
                    Log.e(TAG, "MacBlueTooth " + deviceMAC);


                } else {//No se activo BT, salimos de la app
                    deviceMAC = "00:16:A4:04:99:C4";
                    Log.d(TAG, "Bluetooth: No se a dado ninguna MAC");
                    finish();
                }
                ConfigBT();

                break;

            case REQUEST_SETTINS_PANTALLA://Respuesta de intento de encendido de BT
                if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio

                } else {//No se activo BT, salimos de la app
                    Log.d(TAG, "Error: Pantalla Colores");
                    finish();
                }
                break;

            case REQUEST_PAIR_DEVICE://Respuesta de intento de encendido de BT

                pairDevice(device);

                break;

        }//fin de switch case
    }//fin de onActivityResult

    public void ConfigBT() {

        // Obtenemos el adaptador de bluetooth
        //Boolean success= true;
        AdaptadorBT = BluetoothAdapter.getDefaultAdapter();

        if (AdaptadorBT.isEnabled()) {//Si el BT esta encendido,REQUEST_INICIO_PANTALLA
            if (Servicio_BT == null) {//y el Servicio_BT es nulo, invocamos el Servicio_BT
                Servicio_BT = new ConexionBT(this, mHandler);

                //Conectamos con el dispositivo
                vibrador = (Vibrator) getSystemService(VIBRATOR_SERVICE);
                vibrador.vibrate(100);

                device = AdaptadorBT.getRemoteDevice(deviceMAC);

                Servicio_BT.connect(device);
            }
        } else {
            if (D) Log.e("Setup", "Bluetooth apagado...");
            Intent enableBluetooth = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBluetooth, REQUEST_ENABLE_BT);
        }
    }

    public void pairDevice(BluetoothDevice device) {
        byte[] pinBytes = new byte[]{'1', '2', '3', '4'};

        try {
            Log.d(TAG, "Try to set the PIN");
            Method m = device.getClass().getMethod("setPin", byte[].class);
            m.invoke(device, pinBytes);
            Log.d(TAG, "Success to add the PIN.");
            try {
                device.getClass().getMethod("setPairingConfirmation", boolean.class).invoke(device, true);
                // device.getClass().getMethod("cancelPairingUserInput", boolean.class).invoke(device);

                //   device.getClass().getMethod("setPairingConfirmation", boolean.class).invoke(device, true);
                Log.d(TAG, "Success to setPairingConfirmation.");
            } catch (Exception e) {
                // TODO Auto-generated catch block
                Log.e(TAG, e.getMessage());
                e.printStackTrace();
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
            e.printStackTrace();
        }
    }

    private void buscarBT() {
        final Intent intent = new Intent(this, ListadoDeviceBT.class);
        intent.setAction(Intent.ACTION_MAIN);
        intent.addCategory(Intent.CATEGORY_LAUNCHER);
        startActivityForResult(intent, REQUEST_LIST_BT);

    }

    private void conectadoBT() {

        sendMessage("*?VE$");
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        sendMessage("*?FR$");

        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        sendMessage("*SC30$");
    }

    private void desconectadoBT() {

        if (Servicio_BT != null) Servicio_BT.stop();//Detenemos servicio
        Servicio_BT = null;

        Toast.makeText(getApplicationContext(), "Desconectado de " + mConnectedDeviceName, Toast.LENGTH_LONG).show();

        if (!finAplicacionPromax12) {
            buscarBT();
        }
    }

    public void sendMessage(String message) {
        if (Servicio_BT.getState() == ConexionBT.STATE_CONNECTED) {//checa si estamos conectados a BT
            if (message.length() > 0) {   // checa si hay algo que enviar
                byte[] send = message.getBytes();//Obtenemos bytes del mensaje
                if (D) Log.e(TAG, "Mensaje enviado:" + message);
                Servicio_BT.write(send);     //Mandamos a escribir el mensaje
            }
        } else Toast.makeText(this, "No conectado", Toast.LENGTH_SHORT).show();
    }//fin de sendMessage

    public void showNotification(String mensajeNotificacion) {

        if (activarNotificaciones) {
            Context context = this;
            NotificationCompat.Builder mBuilder =
                    new NotificationCompat.Builder(context)
                            .setSmallIcon(R.drawable.ic_launcher)
                            .setContentTitle(mensajeNotificacion)
                            .setContentText("Toca para acceder a la aplicación")
                            .setDefaults(Notification.DEFAULT_ALL);
            // Creates an explicit intent for an Activity in your app
            final Intent notificationIntent = new Intent(context, ActivityPantallaContelacion.class);
            notificationIntent.setAction(Intent.ACTION_MAIN);
            notificationIntent.addCategory(Intent.CATEGORY_LAUNCHER);
            // The stack builder object will contain an artificial back stack for the
            // started Activity.
            // This ensures that navigating backward from the Activity leads out of
            // your application to the Home screen.

            // Adds the back stack for the Intent (but not the Intent itself)
            PendingIntent resultPendingIntent = PendingIntent.getActivity(context, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);

            mBuilder.setContentIntent(resultPendingIntent);

            mNotificationManager.notify(NOTIF_ID, mBuilder.build());
        }
    }

}
