/*
* @par License info
*
*
* Copyright (C) 2016 Carlos Fernández Moraga
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* This is free software, and you are welcome to redistribute it.
*
*/

package es.promax.promaxbt;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;


public class DibujarLienzoConstelacion extends View {


    float yPuntos;
    float xPuntos;
    private int width;
    private int height;
    private int puntosContelacion = 255;
    private float escalaXpuntos;
    private float escalaYpuntos;
    private Paint paintLineas = new Paint();
    private Paint paintPuntConstelacion = new Paint();
    private float testTextSize = 48f;

    private Promax12 px12;


    private int[][] arrayColisionesConstelacion;


    public DibujarLienzoConstelacion(Context context, AttributeSet attrs, Promax12 promax12) {
        super(context, attrs);
        this.px12 = promax12;
        setArrayColisionesConstelacion(px12.getArrayColisionesConstelacion());
    }

    public DibujarLienzoConstelacion(Context context, Promax12 promax12) {
        super(context);
        this.px12 = promax12;
        setArrayColisionesConstelacion(px12.getArrayColisionesConstelacion());
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        width = w;
        height = h;

        super.onSizeChanged(w, h, oldw, oldh);
    }

    private void RedimensionarSize() {

        yPuntos = height / 8.0f;
        xPuntos = width / 8.0f;

        escalaYpuntos = (height / (float) puntosContelacion);
        escalaXpuntos = (width / (float) puntosContelacion);

        float escala = getResources().getDisplayMetrics().density;

        Log.d("TAG", "Escala " + escala);
        Log.d("TAG", "onSizeChanged: width " + width + ", height " + height);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        canvas.drawColor(Color.BLACK);

        RedimensionarSize();

        pintaEjes(canvas);


        if (px12.getInicializadoPromax12()) {
            pintaPunt(canvas);
        }

    }

    private void pintaEjes(Canvas canvas) {


        paintLineas.setAntiAlias(true);
        paintLineas.setStyle(Style.STROKE);
        paintLineas.setStrokeWidth(1);
        paintLineas.setTextSize(testTextSize);
        paintLineas.setColor(Color.WHITE);


        for (int i = 1; i < 8; i++) {
            //Vertical line -----------------------------
            canvas.drawLine(0, (i * yPuntos), width, (i * yPuntos), paintLineas);
            //Horizontal line --------------------------------
            canvas.drawLine(i * xPuntos, 0, i * xPuntos, height, paintLineas);
        }
    }


    public void pintaPunt(Canvas canvas) {

        paintPuntConstelacion.setAntiAlias(true);
        paintPuntConstelacion.setStyle(Style.STROKE);
        paintPuntConstelacion.setStrokeWidth(escalaXpuntos * 1.5f);

        for (int i = 0; i < 255; i++) {
            for (int j = 0; j < 255; j++) {

                int aux = arrayColisionesConstelacion[i][j];
                if (aux > 0) {

                    paintPuntConstelacion.setColor(aumentarDegradado(aux));

                    canvas.drawPoint(escalaXpuntos * i, escalaYpuntos * j, paintPuntConstelacion);
                }
            }
        }
    }


    private int aumentarDegradado(int numbreColisions) {

        int rojo;
        int verde;
        int azul;
        float numbreMax = 32;
        float transicion = numbreMax / 4;
        int numeroDeImcrementos = 1020;//255*4;
        float incremento = numeroDeImcrementos / numbreMax;


        if (numbreColisions <= (transicion)) {
            rojo = 0;
            verde = (int) (incremento * (numbreColisions % transicion));
            azul = 255;
        } else if (numbreColisions <= (transicion * 2)) {
            rojo = 0;
            verde = 255;
            azul = (int) (255.00 - incremento * (numbreColisions % transicion));
        } else if (numbreColisions <= (transicion * 3)) {
            rojo = (int) (incremento * (numbreColisions % transicion));
            verde = 255;
            azul = 0;
        } else {
            rojo = 255;
            verde = (int) (255.00 - incremento * (numbreColisions % transicion));
            azul = 0;
        }
        return (Color.rgb(rojo, verde, azul));
    }

    public void setArrayColisionesConstelacion(int[][] arrayColisionesConstelacion) {
        this.arrayColisionesConstelacion = arrayColisionesConstelacion;
    }
}