/*
* @par License info
*
*
* Copyright (C) 2016 Carlos Fernández Moraga
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* This is free software, and you are welcome to redistribute it.
*
*/
package es.promax.promaxbt;

import android.app.Activity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.graphics.Point;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.Vibrator;
import android.support.v4.app.NotificationCompat;
import android.support.v4.view.MotionEventCompat;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.LinearLayout;
import android.widget.Toast;

import java.lang.reflect.Method;
import java.util.ArrayList;

import es.promax.bluetooth.ConexionBT;
import es.promax.bluetooth.ListadoDeviceBT;
import es.promax.inicio.PantallaInicioActivity;


public class MainActivity extends Activity {
	
	public static final String TAG = "PX12";
	public static final boolean D = true;
	
    // Tipos de mensaje enviados y recibidos desde el Handler de ConexionBT
    public static final int MESSAGE_ESTADO_CAMBIADO 		= 1;
    public static final int MESSAGE_LEIDO 					= 2;
    public static final int MESSAGE_ESCRITO					= 3;
    public static final int MESSAGE_NOMBRE_DISPOSITIVO 		= 4;
    public static final int MESSAGE_TOAST 					= 5;	        
    public static final int MESSAGE_DESCONECTADO 			= 6;
	public static final int MESSAGE_ERROR_CONEXION 			= 7;
	
    public static final int MENSAJE_RECIBIDO_NOVALIDO 		= 0;
    public static final int MENSAJE_RECIBIDO_ALARMA 		= 1;
    public static final int MENSAJE_RECIBIDO_BATERIA		= 2;
    public static final int MENSAJE_RECIBIDO_TEXTO 			= 3;
    public static final int MENSAJE_RECIBIDO_VERSION 		= 4;
	public static final int MENSAJE_RECIBIDO_MESURA_SCAN = 5;
	public static final int MENSAJE_RECIBIDO_MESURA_SPECTRUM = 6;
	public static final int MENSAJE_RECIBIDO_FREQUENCY = 7;
	public static final int MENSAJE_RECIBIDO_POINT_COSTELATION = 8;
	public static final int MENSAJE_RECIBIDO_DADES_COSTELLATION = 9;
	public static final int MENSAJE_RECIBIDO_BERS_COSTELLATION = 10;
	public static final int MENSAJE_RECIBIDO_MER_COSTELLATION = 11;
	public static final int MENSAJE_RECIBIDO_DBM_EQUIP = 12;


	//-----------------
	public static final String DEVICE_NAME = "device_name";
	public static final String TOAST = "toast";
    //-----------------
    public static final int REQUEST_ENABLE_BT = 107;
    public static final int REQUEST_LIST_BT = 108;
    public static final int REQUEST_INICIO_PANTALLA = 109;
    public static final int REQUEST_SETTINS_PANTALLA = 110;
    public static final int REQUEST_PAIR_DEVICE = 120;
	private static final int NOTIF_ID = 91372;
	private BluetoothDevice device;
	//Nombre del dispositivo conectado
    private String mConnectedDeviceName = null;
	private BluetoothAdapter AdaptadorBT = null;
	//Objeto miembro para el servicio de ConexionBT
	private ConexionBT Servicio_BT = null;
	private String deviceMAC = "00:16:A4:04:99:C4";
	//Vibrador
	private Vibrator vibrador;
	private Boolean activarNotificaciones = false;
	private Boolean inicioAplicacionPromax12 = true;
	private Boolean finAplicacionPromax12 = false;
	private NotificationManager mNotificationManager;
	private DibujarLienzo lienzo;

	private Point puntoInicio1;
	private Point puntoFin1;
    private Point puntoInicio2;
    private Point puntoFin2;
    
    private Promax12 px12 = new Promax12();
	final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {

			switch (msg.what) {
				//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				case MESSAGE_ESCRITO:
					byte[] writeBuf = (byte[]) msg.obj;//buffer de escritura...
					// Construye un String del Buffer
					String writeMessage = new String(writeBuf);
					if (D) Log.e(TAG, "Message_write  =w= " + writeMessage);
					break;
				//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				case MESSAGE_LEIDO:
					//byte[] readBuf = (byte[]) msg.obj;//buffer de lectura...
					//Construye un String de los bytes validos en el buffer
					// String readMessage = new String(readBuf, 0, msg.arg1);
					String readMessage = (String) msg.obj;
					if (msg.arg1 > 0) {
						//  mainText.append(readMessage);
						px12.mensajeRecibido(readMessage);
						if (D) Log.e(TAG, "Message_read   =r= " + readMessage);
					}
					break;
				//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				case MESSAGE_NOMBRE_DISPOSITIVO:
					mConnectedDeviceName = msg.getData().getString(DEVICE_NAME); //Guardamos nombre del dispositivo
					Toast.makeText(getApplicationContext(), "Conectado con " + mConnectedDeviceName, Toast.LENGTH_SHORT).show();
					conectadoBT();
					break;
				//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				case MESSAGE_TOAST:
					Toast.makeText(getApplicationContext(), msg.getData().getString(TOAST),
							Toast.LENGTH_SHORT).show();
					break;
				//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				case MESSAGE_DESCONECTADO:
					if (D) Log.e("Conexion", "DESConectados");
					desconectadoBT();
					showNotification("Bluetooth desconectado");
					break;
				//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				case MESSAGE_ERROR_CONEXION:
					if (D) Log.e("Conexion", "Error Conexion");
					desconectadoBT();
					break;
				//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
			}//FIN DE SWITCH CASE PRIMARIO DEL HANDLER
		}//FIN DE METODO INTERNO handleMessage
	};//Fin de Handler

	@Override
	protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

		if (inicioAplicacionPromax12) {

          	inicioPantallaPresentacion();
			inicioAplicacionPromax12 = false;

        }


		setContentView(R.layout.activity_main);

        puntoInicio1=new Point(-1,-1);
        puntoFin1=new Point(-1,-1);
        puntoInicio2=new Point(-1,-1);
        puntoFin2=new Point(-1,-1);


		//Mensajes barra de estado
		mNotificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);

        LinearLayout layautDibujo = (LinearLayout) findViewById(R.id.linearLayoutLienzo);

        lienzo = new DibujarLienzo(this,px12);

        layautDibujo.addView(lienzo);


        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);


		lienzo.setOnTouchListener(new DibujarLienzo.OnTouchListener() {



			@Override
			public boolean onTouch(View v, MotionEvent event) {


				int action = MotionEventCompat.getActionMasked(event);


				// Get the index of the pointer associated with the action.
				int index = MotionEventCompat.getActionIndex(event);
				int xPos = -1;
				int yPos = -1;

				Log.d(TAG,"The action is " + index);

				switch (action) {
					case MotionEvent.ACTION_DOWN:
						if (index==0){
							puntoInicio1.x = (int)MotionEventCompat.getX(event, index);
							puntoInicio1.y = (int)MotionEventCompat.getY(event, index);

							Log.d(TAG, "puntoInicio1"  + puntoInicio2.x + " "+ puntoFin2.x );
						}


						return true;


					case MotionEvent.ACTION_POINTER_DOWN:


				        for (int size = event.getPointerCount(), i = 0; i < size; i++) {

							if (i>0){

								puntoInicio2.x = (int)MotionEventCompat.getX(event, i);
								puntoInicio2.y = (int)MotionEventCompat.getY(event, i);
						    	   Log.d("TAG", "mecago puntoInicio2 " + puntoInicio2.x + " "+ puntoFin2.x );

							}

				          }


						return true;
					case MotionEvent.ACTION_MOVE:

						if (index==0){
							puntoFin1.x = (int)MotionEventCompat.getX(event, index);
							puntoFin1.y = (int)MotionEventCompat.getY(event, index);

							Log.d(TAG, "puntoInicio1"  + puntoInicio2.x + " "+ puntoFin2.x );


							if (lienzo.getRectSpan().contains(puntoFin1.x, puntoFin1.y)) {

							      lienzo.setPintaLineaFrequency(true, puntoFin1.x);

							      lienzo.invalidate();

							      Log.d("TAG", "mecago ACTION_UP 2 : " + puntoInicio1.x + " "+ puntoFin1.x + " "+ puntoInicio2.x + " "+ puntoFin2.x + " "  );


							}

						} else {

							lienzo.setPintaLineaFrequency(false, puntoFin1.x);
						}

					     return true;


					case MotionEvent.ACTION_UP:

						if (index==0){
							puntoFin1.x = (int)MotionEventCompat.getX(event, index);
							puntoFin1.y = (int)MotionEventCompat.getY(event, index);

							Log.d(TAG, "puntoInicio1"  + puntoInicio2.x + " "+ puntoFin2.x );

						     if	(lienzo.getRectSpan().contains(puntoFin1.x ,puntoFin1.y)&& lienzo.isPintaLineaFrequency()){

						    	  String auxEnviar ="";

							      lienzo.setPintaLineaFrequency(true, puntoFin1.x);

							      px12.setFrequencyKhz(lienzo.getnewLineaFrequency());

							      lienzo.invalidate();

							      auxEnviar="*FR"+ String.format("%06X", px12.getFrequencyKhz() & 0x0FFFFF)+"$";

							      sendMessage(auxEnviar);

							      Log.d("TAG", "mecago ACTION_UP 2 : " + puntoInicio1.x + " "+ puntoFin1.x + " "+ puntoInicio2.x + " "+ puntoFin2.x + " " + auxEnviar );


							 }
						}

						int distanciaY =  Math.abs(puntoInicio1.y-puntoFin1.y);

					     if	(lienzo.getRectReferenLevel().contains(puntoFin1.x ,puntoFin1.y) && (distanciaY> lienzo.getDistanciaMinimaX()) ){

					    	 String auxEnviar ="";

						      if(puntoInicio1.y>puntoFin1.y){
						    	  int aux = px12.getReferenLevel();
						    	  aux--;
						    	  if (aux<1) aux=1;
						    	  px12.setReferenLevel(aux);
						    	  auxEnviar="*BR01$";
						      }

						      if(puntoInicio1.y<puntoFin1.y){
						    	  int aux = px12.getReferenLevel();

								  aux++;
								  if (aux>8) aux=8;
						    	  px12.setReferenLevel(aux);
						    	  auxEnviar="*BR-1$";
						      }
						      sendMessage(auxEnviar);

							 lienzo.invalidate();

						 }

						lienzo.setPintaLineaFrequency(false, puntoFin1.x);


						Log.d(TAG, "MotionEvent.ACTION_UP " + " " + puntoFin1.x + " " + puntoFin1.y);

					case MotionEvent.ACTION_POINTER_UP:

				        for (int size = event.getPointerCount(), i = 0; i < size; i++) {

							if (i>0){

								puntoFin2.x = (int) MotionEventCompat.getX(event, i);
								puntoFin2.y = (int)MotionEventCompat.getY(event, i);
						    	   Log.d("TAG", "mecago puntoFin2 " + puntoInicio2.x + " "+ puntoFin2.x );

							} else {
								puntoFin1.x = (int)MotionEventCompat.getX(event, i);
							    	   puntoFin1.y = (int)MotionEventCompat.getY(event, i);
							    	   Log.d("TAG", "mecago puntoFin1 " + puntoInicio1.x + " "+ puntoFin1.x );
								}
				          }

						//if (event.getPointerCount() > 1){

						int distanciaX1 =  Math.abs(puntoInicio1.x-puntoFin1.x);
						int distanciaX2 =  Math.abs(puntoInicio2.x-puntoFin2.x);

						if (lienzo.getRectGrafica().contains(puntoFin1.x, puntoFin1.y) &&
								lienzo.getRectGrafica().contains(puntoFin2.x, puntoFin2.y) && (distanciaX1 > lienzo.getDistanciaMinimaX())
								&& (distanciaX2> lienzo.getDistanciaMinimaX())){

							String auxEnviar = "";

							if (puntoInicio1.x < puntoFin1.x && puntoInicio2.x > puntoFin2.x) {
								int aux = px12.getSpam();
								    	  aux--;
								    	  if (aux<-1) aux=-1;
								    	  px12.setSpam(aux);
								if (aux == -1) aux = 6;    //FULL SPAN ES 6
								auxEnviar = "*BS0" + Integer.toString(aux) + "$";
							}

							if (puntoInicio1.x > puntoFin1.x && puntoInicio2.x < puntoFin2.x) {
								int aux = px12.getSpam();

								aux++;
								if (aux>5) aux=5;
								    	  px12.setSpam(aux);
								    	  auxEnviar="*BS0"+Integer.toString(aux)+"$";
								      }
								      sendMessage(auxEnviar);

							Log.d("TAG", "mecago ACTION_UP 2 : " + puntoInicio1.x + " " + puntoFin1.x + " " + puntoInicio2.x + " " + puntoFin2.x + " " + auxEnviar);

							     }


						lienzo.invalidate();
						Log.d("CV", "Other point up");


						return true;


					case (MotionEvent.ACTION_CANCEL):
						Log.d("TAG", "La accion ha sido CANCEL");
				        return true;
				    case (MotionEvent.ACTION_OUTSIDE):
				        Log.d("TAG",
				                "La accion ha sido fuera del elemento de la pantalla");
				        return true;

				    default:
				    	return false;
				       // return super.onTouchEvent(event);
				    }
			}

		});

        startTimerThread();
    }
    
    private void startTimerThread() {
        final Handler handler = new Handler();

		Runnable runnable = new Runnable() {
			@Override
			public void run() {
				while (true) {
					try {
						px12.gestionListaDeMensajesRecibido();
                        Thread.sleep(5);
					} catch (InterruptedException e) {
						e.printStackTrace();
                    }
                    handler.post(new Runnable(){
                        @Override
						public void run() {
                        	gestionarMensajeRecibido();
                    }
                });
                }
            }
        };
        new Thread(runnable).start();
    }
    
 public  void gestionarMensajeRecibido() {


	 ArrayList<Integer> listadoMensajes = px12.getListadoMensajes();


	 ArrayList<Integer> arrayListCopia = new ArrayList<Integer>(listadoMensajes);


		listadoMensajes.clear();


	 while (arrayListCopia.remove(null)) {
	 }


	 if (!arrayListCopia.isEmpty()) {

    		arrayListCopia.remove(null);

		 for (Integer i : arrayListCopia) {

			 switch (i) {

				 case MENSAJE_RECIBIDO_BATERIA:
					 try{
        	    		  lienzo.invalidate();
        	    	  }
        	    	  catch(NumberFormatException ex){
        	    		Log.d(TAG, "No es un número, es una cadena de texto.");
        	    	  }
					 break;
				 case MENSAJE_RECIBIDO_TEXTO:
        	          break;
        	      case MENSAJE_RECIBIDO_VERSION:
        	           break;
        	      case MENSAJE_RECIBIDO_NOVALIDO:
        	           break;
        	      case MENSAJE_RECIBIDO_ALARMA:
        	    	   showNotification("Alarma activada");
        	    	   lienzo.invalidate();
        	    	   vibrador.vibrate(100);
					  break;
				 case MENSAJE_RECIBIDO_MESURA_SCAN:
					 px12.setInicializadoPromax12(true);
    	    	  		px12.setPantalla(MENSAJE_RECIBIDO_MESURA_SCAN);
    	    	  		lienzo.invalidate();
    	    	  		break;
				 case MENSAJE_RECIBIDO_MESURA_SPECTRUM:
					 px12.setInicializadoPromax12(true);
					 px12.setPantalla(MENSAJE_RECIBIDO_MESURA_SPECTRUM);
					  lienzo.invalidate();
					  break;
        	      }
              }
              //La borramos
		 arrayListCopia.clear();
		 //arrayListCopia

	 }
 }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

	@Override
	protected void onRestart() {
		super.onRestart();
        activarNotificaciones=false;
        Log.d(TAG, "NotificationTest: OnRestart");
    }

    @Override protected void onResume() {
        super.onResume();
        Log.d(TAG, "NotificationTest: OnResume");
    }

	@Override
	protected void onStop() {
		super.onStop();
        activarNotificaciones=true;
        Log.d(TAG, "NotificationTest: OnStop");
    }

	@Override
	public  void onStart() {
    super.onStart();
    }

    @Override
    public void onDestroy(){
    	 super.onDestroy();

		activarNotificaciones = false;
		finAplicacionPromax12 = true;

    	 mNotificationManager.cancel(NOTIF_ID);

		if (Servicio_BT != null) Servicio_BT.stop();//Detenemos servicio
		Servicio_BT=null;
		finish();
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        if (id == R.id.action_settings) {
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

	//-----  Bluetooth ------

	private void inicioPantallaPresentacion() {
		Intent intent = new Intent(MainActivity.this, PantallaInicioActivity.class);
		startActivityForResult(intent, REQUEST_INICIO_PANTALLA);
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
    	super.onActivityResult(requestCode, resultCode, data);
  	  //Una vez que se ha realizado una actividad regresa un "resultado"...
  	      switch (requestCode) {

			  case REQUEST_ENABLE_BT://Respuesta de intento de encendido de BT
				  if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio
  	                	ConfigBT();

					} else {//No se activo BT, salimos de la app
						Log.d(TAG, "Bluetooth: el usuario NO acepta encenderlo");
					  finish();}
  	             break;

			  case REQUEST_INICIO_PANTALLA://Respuesta de intento de encendido de BT
				  if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio
  	                	buscarBT();

					} else {//No se activo BT, salimos de la app
						Log.d(TAG, "Error: Pantalla inicio");
					  finish();}
  	             break;

			  case REQUEST_LIST_BT://Respuesta de intento de encendido de BT
				  if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio
  	                	deviceMAC = data.getStringExtra("deviceMAC");
  	                	Log.e(TAG, "MacBlueTooth " + deviceMAC);


					} else {//No se activo BT, salimos de la app
						deviceMAC = "00:16:A4:04:99:C4";
					  Log.d(TAG, "Bluetooth: No se a dado ninguna MAC");
  	                    finish();}
  	              ConfigBT();

				  break;

			  case REQUEST_SETTINS_PANTALLA://Respuesta de intento de encendido de BT
				  if (resultCode == Activity.RESULT_OK) {//BT esta activado,iniciamos servicio

					} else {//No se activo BT, salimos de la app
						Log.d(TAG, "Error: Pantalla Colores");
					  finish();}
  	             break;

  	            case REQUEST_PAIR_DEVICE://Respuesta de intento de encendido de BT

					pairDevice(device);

  	             break;

		  }//fin de switch case
	}//fin de onActivityResult

	public void ConfigBT() {

		// Obtenemos el adaptador de bluetooth
		//Boolean success= true;
        AdaptadorBT = BluetoothAdapter.getDefaultAdapter();

		if (AdaptadorBT.isEnabled()) {//Si el BT esta encendido,REQUEST_INICIO_PANTALLA
			if (Servicio_BT == null) {//y el Servicio_BT es nulo, invocamos el Servicio_BT
				Servicio_BT = new ConexionBT(this, mHandler);

				//Conectamos con el dispositivo
				vibrador = (Vibrator) getSystemService(VIBRATOR_SERVICE);
				vibrador.vibrate(100);

              device = AdaptadorBT.getRemoteDevice(deviceMAC);

				Servicio_BT.connect(device);
			}
		} else{ if(D) Log.e("Setup", "Bluetooth apagado...");
     Intent enableBluetooth = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableBluetooth, REQUEST_ENABLE_BT);
		}
	}
    
    public void pairDevice(BluetoothDevice device)
    {
        byte[] pinBytes = new byte[] {'1', '2', '3', '4'};

        try {
            Log.d(TAG, "Try to set the PIN");
            Method m = device.getClass().getMethod("setPin", byte[].class);
            m.invoke(device, pinBytes);
            Log.d(TAG, "Success to add the PIN.");
            try {
         	   device.getClass().getMethod("setPairingConfirmation", boolean.class).invoke(device, true);
                  Log.d(TAG, "Success to setPairingConfirmation.");
              } catch (Exception e) {
                  // TODO Auto-generated catch block
                  Log.e(TAG, e.getMessage());
                  e.printStackTrace();
			}
		} catch (Exception e) {
			Log.e(TAG, e.getMessage());
            e.printStackTrace();
		}
	}

	private  void buscarBT() {
    	final Intent intent = new Intent (this, ListadoDeviceBT.class);
		intent.setAction(Intent.ACTION_MAIN);
		intent.addCategory(Intent.CATEGORY_LAUNCHER);
		startActivityForResult(intent,REQUEST_LIST_BT);

	}

	private  void conectadoBT() {


        sendMessage("*?VE$");
    	try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        sendMessage("*?BS$");
    	try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        sendMessage("*?BR$");
    	try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        sendMessage("*?FR$");


		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    private  void desconectadoBT() {


		if (Servicio_BT != null) Servicio_BT.stop();//Detenemos servicio
		Servicio_BT=null;

	    Toast.makeText(getApplicationContext(), "Desconectado de "+ mConnectedDeviceName, Toast.LENGTH_LONG).show();

		if (!finAplicacionPromax12) {
			buscarBT();
	    }
    }
    
    public  void sendMessage(String message) {
		if (Servicio_BT.getState() == ConexionBT.STATE_CONNECTED) {//checa si estamos conectados a BT
			if (message.length() > 0) {   // checa si hay algo que enviar
				byte[] send = message.getBytes();//Obtenemos bytes del mensaje
				if (D) Log.e(TAG, "Mensaje enviado:" + message);
				Servicio_BT.write(send);     //Mandamos a escribir el mensaje
			}
        	} else Toast.makeText(this, "No conectado", Toast.LENGTH_SHORT).show();
    		}//fin de sendMessage

	public  void showNotification(String mensajeNotificacion) {

		if (activarNotificaciones) {
			Context context=this;
            NotificationCompat.Builder mBuilder =
                    new NotificationCompat.Builder(context)
                    .setSmallIcon(R.drawable.ic_launcher)
                    .setContentTitle(mensajeNotificacion)
                    .setContentText("Toca para acceder a la aplicación")
                    .setDefaults(Notification.DEFAULT_ALL);
            // Creates an explicit intent for an Activity in your app
            final Intent notificationIntent = new Intent(context, MainActivity.class);
            notificationIntent.setAction(Intent.ACTION_MAIN);
            notificationIntent.addCategory(Intent.CATEGORY_LAUNCHER);
            // The stack builder object will contain an artificial back stack for the
            // started Activity.
            // This ensures that navigating backward from the Activity leads out of
            // your application to the Home screen.

            // Adds the back stack for the Intent (but not the Intent itself)
            PendingIntent resultPendingIntent = PendingIntent.getActivity(context,0,notificationIntent,PendingIntent.FLAG_UPDATE_CURRENT);

            mBuilder.setContentIntent(resultPendingIntent);

            //NotificationManager mNotificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

            // mId allows you to update the notification later on.

			mNotificationManager.notify(NOTIF_ID, mBuilder.build());
		}
    }
  	        
}
